@@ -20,9 +20,12 @@ from typing import (
     cast,
     overload,
     runtime_checkable,
+    Literal,
+    TypeGuard,
+    Generic,
 )
 
-from astream import NoValueSentinel, SentinelType, ensure_async_iterator, ensure_coro_fn
+from astream import NoValue, SentinelType, ensure_async_iterator, ensure_coro_fn, NoValueT
 
 _T = TypeVar("_T")
 _U = TypeVar("_U")
@@ -291,6 +294,41 @@ async def arange(start: int, stop: int | None = None, step: int = 1) -> AsyncIte
         yield i
 
 
+# class MergedAsyncIterator(Generic[_T], AsyncIterator[_T]):
+#     """A protocol for a merged stream of items."""
+#
+#     merged_iterables: tuple[Iterable[_T] | AsyncIterable[_T], ...]
+#
+#     def __init__(self, *streams: Iterable[_T] | AsyncIterable[_T]) -> None:
+#         self.merged_iterables = streams
+#
+#         self._futs: dict[asyncio.Future[_T], AsyncIterator[_T]] = {}
+#         for it in self.merged_iterables:
+#             async_it = ensure_async_iterator(it)
+#             fut = asyncio.ensure_future(anext(async_it))
+#             self._futs[fut] = async_it
+#
+#     def __aiter__(self) -> MergedAsyncIterator[_T]:
+#         return self
+#
+#     async def __anext__(self) -> _T:
+#         if not self._futs:
+#             raise StopAsyncIteration
+#         done, _ = await asyncio.wait(self._futs, return_when=asyncio.FIRST_COMPLETED)
+#         for done_fut in done:
+#             try:
+#                 item = done_fut.result()
+#                 fut = asyncio.ensure_future(anext(self._futs[done_fut]))
+#                 self._futs[fut] = self._futs[done_fut]
+#                 return item
+#             except StopAsyncIteration:
+#                 return await anext(self)
+#             finally:
+#                 if done_fut in self._futs:
+#                     del self._futs[done_fut]
+#         raise StopAsyncIteration
+
+
 def amerge(*async_iters: Iterable[_T] | AsyncIterable[_T]) -> AsyncIterator[_T]:
     """Merge multiple iterables or async iterables into one, yielding items as they are received.
 
@@ -350,7 +388,7 @@ def amerge(*async_iters: Iterable[_T] | AsyncIterable[_T]) -> AsyncIterator[_T]:
 async def ascan(
     fn: Callable[[_T, _U], Coroutine[Any, Any, _T]] | Callable[[_T, _U], _T],
     iterable: AsyncIterable[_U],
-    initial: _T | SentinelType = NoValueSentinel,
+    initial: _T | NoValueT = NoValue,
 ) -> AsyncIterator[_T]:
     """An asynchronous version of `scan`.
 
@@ -376,7 +414,7 @@ async def ascan(
     _fn_async = ensure_coro_fn(fn)
     _it_async = ensure_async_iterator(iterable)
 
-    if initial is NoValueSentinel:
+    if isinstance(initial, NoValueT):
         initial = await anext(_it_async)  # type: ignore
     crt = cast(_T, initial)
 
@@ -480,10 +518,175 @@ async def azip_longest(
     """An asynchronous version of `zip_longest`."""
     async_iterables = [ensure_async_iterator(it) for it in iterables]
     while True:
-        items = await asyncio.gather(*(anext(it, NoValueSentinel) for it in async_iterables))
-        if all(item is NoValueSentinel for item in items):
+        items = await asyncio.gather(*(anext(it, NoValue) for it in async_iterables))
+        if all(item is NoValue for item in items):
             break
-        yield tuple(item if item is not NoValueSentinel else fillvalue for item in items)
+        yield tuple(item if item is not NoValue else fillvalue for item in items)
+
+
+async def bytes_stream_split_separator(
+    stream: AsyncIterable[bytes],
+    separator: bytes = b"\n",
+    strip_characters: tuple[bytes, ...] = (b"\r", b"\n", b"\t", b" ", b"\x00", b"\x0b", b"\x0c"),
+) -> AsyncIterator[bytes]:
+    """Splits a stream of bytes by a separator.
+
+    Args:
+        stream: The stream of bytes.
+        separator: The separator to split by.
+        strip_characters: The characters to strip from the end/beginning of the split.
+
+    Yields:
+        The split bytes.
+
+    Examples:
+        >>> from astream import stream
+        >>> async def demo_bytes_stream_split_separator():
+        ...     async for it in bytes_stream_split_separator(
+        ...         stream([b"hello", b"world", b"!"]),
+        ...         b"o",
+        ...     ):
+        ...         print(it)
+        >>> asyncio.run(demo_bytes_stream_split_separator())
+        b'hell'
+        b'w'
+        b'rld!'
+    """
+
+    # b"\x00" is a null byte, which is used to terminate strings in C.
+    # b"\x0b" and b"\x0c" are vertical and form feed characters, which are used to terminate
+    # strings in some languages. They are also used to separate pages in some terminal emulators.
+
+    strip_characters_str = b"".join(strip_characters)
+    buf = bytearray()
+    async for chunk in stream:
+        buf.extend(chunk)
+        while True:
+            line, sep, remaining = buf.partition(separator)
+            if sep:
+                yield bytes(line.strip(strip_characters_str))
+                buf = bytearray(remaining)
+            else:
+                break
+    yield bytes(buf.strip(strip_characters_str))
+
+
+_AsyncIterableT = TypeVar("_AsyncIterableT", bound=AsyncIterable[Any])
+
+
+class AsyncIteratorWithExceptionHandler(Generic[_T], AsyncIterable[_T]):
+    """An async iterator that handles exceptions.
+
+    Args:
+        iterable: The async iterable to wrap.
+        exception_handler: The exception handler to use.
+
+    The handler is called with the exception and the async iterator. If it returns a value, that
+    value is yielded. If it returns `NoValue`, no value is yielded, but the iteration continues.
+    If it raises an exception, that exception is raised.
+    """
+
+    def __init__(
+        self,
+        iterable: AsyncIterable[_T] | Iterable[_T],
+        exception_handler: Callable[
+            [BaseException, Iterable[_T] | AsyncIterable[_T]], _T | NoValueT
+        ]
+        | Callable[[BaseException, Iterable[_T] | AsyncIterable[_T]], _CoroT[_T | NoValueT]],
+    ) -> None:
+        self._iterator = aiter(ensure_async_iterator(iterable))
+        self._iterable = iterable
+
+        _async_handler = ensure_coro_fn(exception_handler)
+        self._exception_handler = _async_handler
+
+    def __aiter__(self) -> AsyncIteratorWithExceptionHandler[_T]:
+        return self
+
+    async def _anext(self) -> _T:
+        try:
+            return await anext(self._iterator)
+        except StopAsyncIteration:
+            print("StopAsyncIteration raised")
+            raise StopAsyncIteration
+        except BaseException as exc:
+            if isinstance(exc, StopAsyncIteration):
+                print("StopAsyncIteration in BaseException")
+                raise StopAsyncIteration
+            else:
+                result = await self._exception_handler(exc, self._iterable)
+                if isinstance(result, NoValueT):
+                    return await self._anext()
+                else:
+                    return cast(_T, result)
+
+    async def __anext__(self) -> _T:
+        return await self._anext()
+
+
+@overload
+def with_exc_handler(
+    iterable: Iterable[_T] | AsyncIterable[_T],
+    handler: Callable[[BaseException, Iterable[_T] | AsyncIterable[_T]], _CoroT[_T | NoValueT]],
+) -> AsyncIterator[_T]:
+    ...
+
+
+@overload
+def with_exc_handler(
+    iterable: Iterable[_T] | AsyncIterable[_T],
+    handler: Callable[[BaseException, Iterable[_T] | AsyncIterable[_T]], _T | NoValueT],
+) -> AsyncIterator[_T]:
+    ...
+
+
+def with_exc_handler(
+    iterable: Iterable[_T] | AsyncIterable[_T],
+    handler: Callable[[BaseException, Iterable[_T] | AsyncIterable[_T]], _CoroT[_T | NoValueT]]
+    | Callable[[BaseException, Iterable[_T] | AsyncIterable[_T]], _T | NoValueT],
+) -> AsyncIterator[_T]:
+    """Wraps an async iterator with an exception handler.
+
+    The handler is called with the exception and the async iterator. If it returns a value, that
+    value is yielded. If it returns `NoValue`, no value is yielded, but the iteration continues.
+    If it raises an exception, that exception is raised.
+
+    Args:
+        iterable: The async iterator.
+        handler: The exception handler.
+
+    Yields:
+        The yielded values.
+
+    Examples:
+        >>> async def demo_with_exc_handler():
+        ...     async for it in with_exc_handler(arange(5), print):
+        ...         print(it)
+        >>> asyncio.run(demo_with_exc_handler())
+        0
+        1
+        2
+        3
+        4
+    """
+    # return AsyncIteratorWithExceptionHandler(iterable, handler)
+    _async_handler = ensure_coro_fn(handler)
+
+    async def _aiter() -> AsyncIterator[_T]:
+        ait = aiter(iterable)
+        while True:
+            try:
+                yield await anext(ait)
+            except (StopAsyncIteration, StopIteration, GeneratorExit):
+                raise
+            except BaseException as exc:
+                result = await _async_handler(exc, iterable)
+                if isinstance(result, NoValueT):
+                    continue
+                else:
+                    yield cast(_T, result)
+
+    return _aiter()
 
 
 __all__ = (
@@ -505,6 +708,8 @@ __all__ = (
     "atee",
     "azip",
     "azip_longest",
+    "bytes_stream_split_separator",
+    "with_exc_handler",
 )
 
 if __name__ == "__main__":
